--[[
	MiniMount
	Version 1.4a
]]

MiniMount_Version = "1.4a";

MiniMount_Mounts_Slow = {};
MiniMount_Mounts_Regular = {};
MiniMount_Mounts_Epic = {};
MiniMount_Mounts_FlyRegular = {};
MiniMount_Mounts_FlyEpic = {};
MiniMount_Mounts_Unknown = {};
MiniMount_ReportedMounts = {};

local MiniMount_RidingSkill = 1;
local Reported = false;
local CurrentMount = nil;
local NextMount = nil;
local CurrentContinent = GetCurrentMapContinent();

-- Message zones
function MiniMount_Msg(msg, channel, debug)
	if (msg == nil) then
		return;
	end
	if debug == nil then
		debug = 10;
	end
	if channel ~= nil then
		channel = strlower(channel);
	end

	if (channel == "error") then
		DEFAULT_CHAT_FRAME:AddMessage(RED_FONT_COLOR_CODE..MINIMOUNT_TITLE.." "..MINIMOUNT_ERROR..": |r"..msg);
	elseif (channel == "debug") then
		DEFAULT_CHAT_FRAME:AddMessage(LIGHTYELLOW_FONT_COLOR_CODE..MINIMOUNT_TITLE.." "..MINIMOUNT_DEBUG..": |r"..msg);
	else
		DEFAULT_CHAT_FRAME:AddMessage(GREEN_FONT_COLOR_CODE..MINIMOUNT_TITLE..": |r"..msg);
	end
end

-- Loading Function
function MiniMount_Load(self)
	self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("COMPANION_LEARNED");
	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_UPDATE");
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")

	-- Slash Commands
	SlashCmdList["MiniMount"] = MiniMount_CommandLine;
	SLASH_MiniMount1 = "/minimount";
	SLASH_MiniMount2 = "/mount";
	SLASH_MiniMount3 = "/mm";

	if (not MiniMount_Options) then
		MiniMount_Options = {
			["DefaultMode"] = "Auto", -- Other values: "Regular", "Epic", "FlyRegular", FlyEpic"
		}
	end
end

function MiniMount_OnEvent(self,event, a1, ...)
	if event == "VARIABLES_LOADED" then
		MiniMount_GetRidingSkill();
		MiniMount_GetNextMount();
		MiniMount_Macro();
		MiniMount_Msg(MINIMOUNT_VERSION.." "..MiniMount_Version.." "..MINIMOUNT_NOWLOADED);
	elseif event == "ZONE_CHANGED_NEW_AREA" and GetCurrentMapContinent() ~= CurrentContinent then
		CurrentContinent = GetCurrentMapContinent();
	elseif event == "COMPANION_LEARNED" then
		MiniMount_ReportedMounts = {};
		MiniMount_BuildMountArrays();
		MiniMount_GetNextMount();
	elseif event == "MERCHANT_SHOW" or event == "MERCHANT_UPDATE" then
		MiniMount_MerchantColorWindow();
	elseif event == "PLAYER_ENTRING_WORLD" then
		MiniMount_Macro();
	end
end

function MiniMount_CommandLine(cmd)
	-- Force lowercase so we don't have to double-up our options! Also makes it more user-proof.
	cmd = strlower(cmd);

	if (cmd == "minimap off") then
		MiniMountMinimapButton:Hide();
		MiniMount_Msg("Hiding minimap button.");
	elseif (cmd == "minimap on") then
		MiniMountMinimapButton:Show();
		MiniMount_Msg("Showing minimap button.");
	elseif (cmd == "minimap") then
		if (MiniMountMinimapButton:IsVisible()) then
			MiniMountMinimapButton:Hide();
			-- MiniMount_Msg("Hiding minimap button.");
		else
			MiniMountMinimapButton:Show();
			-- MiniMount_Msg("Showing minimap button.");
		end
	elseif (cmd == "reg") or (cmd == "regular")  then
		MiniMount_Mount("Regular");
	elseif (cmd == "epic") then
		MiniMount_Mount("Epic");
	elseif (cmd == "fly reg") or (cmd == "fly regular") then
		MiniMount_Mount("FlyRegular");
	elseif (cmd == "fly epic") then
		MiniMount_Mount("FlyEpic");
	elseif (cmd == "fly") then
		MiniMount_Mount("Fly");
	elseif (cmd == "random") then
		MiniMount_Mount("Random");
	elseif (cmd == nil) or (cmd == "") then
		MiniMount_Mount();
	else
		MiniMount_Msg(MINIMOUNT_ERROR_WRONGOPTION,"error")
	end
end

function MiniMount_Mount(mode)
	if (mode ~= nil) and (mode ~= "") then
		MiniMount_GetNextMount(mode);
	end

	if (not IsFlying()) and (not IsSwimming()) then
		-- local _, MountName = GetCompanionInfo("MOUNT",NextMount);
		-- MiniMount_Msg("Summoning "..MountName.."...");
		MiniMount_MountPrepare();
		CurrentMount = NextMount;
		CallCompanion("MOUNT", CurrentMount);
		MiniMount_GetNextMount();
	else
		if IsFlying() then
			MiniMount_Msg(MINIMOUNT_ERROR_ISFLYING,"error");
		elseif IsSwimming() then
			MiniMount_Msg(MINIMOUNT_ERROR_ISSWIMMING,"error");
		end
	end
end

function MiniMount_GetNextMount(mode)
	MiniMount_BuildMountArrays();
	local MountNum = GetNumCompanions("MOUNT");
	local MountList = {};
	SetMapToCurrentZone();
	CurrentContinent = GetCurrentMapContinent();
	local czone = GetRealZoneText();
	MiniMount_GetRidingSkill();

	if MountNum == 0 then
		MiniMount_Msg(MINIMOUNT_ERROR_NOMOUNTS,"error");
		return;
	end

	if MiniMount_RidingSkill < 75 then
		MiniMount_Msg(MINIMOUNT_ERROR_SKILLNOTHIGHENOUGH,"error");
		return;
	end

	if (not mode) then
		mode = MiniMount_Options["DefaultMode"];
	end

	if (not mode) then
		mode = "Auto";
	end

	local rand = {
		[1] = "Auto",
	}
	if #MiniMount_Mounts_Slow > 0 then
		tinsert(rand, (#rand + 1), "Slow");
	end
	if #MiniMount_Mounts_Regular > 0 then
		tinsert(rand, (#rand + 1), "Regular");
	end
	if #MiniMount_Mounts_Epic > 0 then
		tinsert(rand, (#rand + 1), "Epic");
	end
	if (CurrentContinent == 3) or ((CurrentContinent == 4) and (UnitLevel("player") >= 77)) then
		if #MiniMount_Mounts_FlyRegular > 0 then
			tinsert(rand, (#rand + 1), "FlyRegular");
		end
		if #MiniMount_Mounts_FlyEpic > 0 then
			tinsert(rand, (#rand + 1), "FlyEpic");
		end
	end
	if #MiniMount_Mounts_Unknown > 0 then
		tinsert(rand, (#rand + 1), "Unknown");
	end

	if mode == "Random" then
		mode = rand[random(1,#rand)];
	end

	if mode == "Auto" then
		if MiniMount_RidingSkill == 75 then
			mode = "Regular";
		elseif MiniMount_RidingSkill >= 150 then
			mode = "Epic";
		end

		if (CurrentContinent == 3) or ((CurrentContinent == 4) and (UnitLevel("player") >= 77)) then
			if (MiniMount_RidingSkill == 225) then
				mode = "FlyRegular";
			elseif (MiniMount_RidingSkill == 300) then
				mode = "FlyEpic";
			end
		end
	end

	if (mode == "Fly") then
		if (MiniMount_RidingSkill == 225) then
			mode = "FlyRegular";
		elseif (MiniMount_RidingSkill == 300) then
			mode = "FlyEpic";
		else
			if MiniMount_RidingSkill == 75 then
				mode = "Regular";
			elseif MiniMount_RidingSkill >= 150 then
				mode = "Epic";
			end
		end
	end

	if (#getglobal("MiniMount_Mounts_"..mode) == 0) then
		MiniMount_Msg(getglobal("MINIMOUNT_NOMOUNTS_"..strupper(mode)),"error");
		return;
	end

	for type, v in pairs(MiniMount_MountZones) do
		for mountid,v2 in pairs(MiniMount_MountZones[type]) do
			local cdb = getglobal("MiniMount_Mounts_"..type);
			local hasvar = false;
			for x=1,#cdb do
				local _, _, MountSID = GetCompanionInfo("MOUNT",cdb[x]);
				if tostring(MountSID) == mountid then
					hasvar = x;
				end
			end
			if hasvar ~= false then	
				local iszone = false;
				for x=1,#v2 do
					if v2[x] == czone then
						iszone = true;
					end
				end
				if iszone == false then
					tremove(cdb, hasvar);
				end
			end
		end
	end

	MountList = getglobal("MiniMount_Mounts_"..mode);
	local chk_flye, chk_flys, chk_ge, chk_gs = nil, nil, nil, nil;
	while (type(MountList) == nil) do
		if (CurrentContinent == 3) or ((CurrentContinent == 4) and (UnitLevel("player") >= 77)) then
			if chk_flye == nil then
				if (#MiniMount_Mounts_FlyEpic > 0) then
					MountList = MiniMount_Mounts_FlyEpic;
				else
					chk_flye = 1;
				end
			elseif chk_flys == nil then
				if (#MiniMount_Mounts_FlyRegular > 0) then
					MountList = MiniMount_Mounts_FlyRegular;
				else
					chk_flys = 1;
				end
			elseif chk_ge == nil then
				if (#MiniMount_Mounts_Epic > 0) then
					MountList = MiniMount_Mounts_Epic;
				else
					chk_ge = 1;
				end
			else
				if (#MiniMount_Mounts_Regular > 0) then
					MountList = MiniMount_Mounts_Regular;
				else
					chk_gs = 1;
				end
			end
		else
			if chk_ge == nil then
				if (#MiniMount_Mounts_Epic > 0) then
					MountList = MiniMount_Mounts_Epic;
				else
					chk_ge = 1;
				end
			elseif chk_gs == nil then
				if (#MiniMount_Mounts_Regular > 0) then
					MountList = MiniMount_Mounts_Regular;
				else
					chk_ge = 1;
				end
			end
		end
		if chk_flye == 1 and chk_flys == 1 and chk_ge == 1 and chk_gs == 1 then
			MiniMount_Msg(MINIMOUNT_ERROR_MOUNTDATABASEEMPTY,"error");
			return;
		end
	end

	local randnum = random(1,#MountList);
	NextMount = MountList[randnum];
	MiniMount_Timer("Macro",1,MiniMount_Macro);
end

function MiniMount_BuildMountArrays()
	local num = GetNumCompanions("MOUNT");

	if num == 0 then
		return;
	end

	local mymounts = {};

	for x=1, num do
		local _, MountName, MountSID = GetCompanionInfo("MOUNT",x);
		
		for y=1,#MiniMount_Mounts["Slow"] do
			if MiniMount_Mounts["Slow"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Slow, (#MiniMount_Mounts_Slow+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Regular"] do
			if MiniMount_Mounts["Regular"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Epic"] do
			if MiniMount_Mounts["Epic"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["FlyRegular"] do
			if MiniMount_Mounts["FlyRegular"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_FlyRegular, (#MiniMount_Mounts_FlyRegular+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["FlyEpic"] do
			if MiniMount_Mounts["FlyEpic"][y] == tostring(MountSID) then
				tinsert(MiniMount_Mounts_FlyEpic, (#MiniMount_Mounts_FlyEpic+1), x);
			end
		end
		for y=1,#MiniMount_Mounts["Shifting"] do
			if MiniMount_Mounts["Shifting"][y] == tostring(MountSID) then
				if MiniMount_RidingSkill < 150 then
					tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
				else
					tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
				end
			end
		end
		for y=1,#MiniMount_Mounts["FlyShifting"] do
			if MiniMount_Mounts["FlyShifting"][y] == tostring(MountSID) then
				if MiniMount_RidingSkill < 150 then
					tinsert(MiniMount_Mounts_Regular, (#MiniMount_Mounts_Regular+1), x);
				else
					tinsert(MiniMount_Mounts_Epic, (#MiniMount_Mounts_Epic+1), x);
				end
				if MiniMount_RidingSkill < 300 then
					tinsert(MiniMount_Mounts_FlyRegular, (#MiniMount_Mounts_FlyRegular+1), x);
				else
					tinsert(MiniMount_Mounts_FlyEpic, (#MiniMount_Mounts_FlyEpic+1), x);
				end
			end
		end

		for type, v in pairs(MiniMount_MountZones) do
			for mountid, v2 in pairs(MiniMount_MountZones[type]) do
				if mountid == tostring(MountSID) then
					local db = getglobal("MiniMount_Mounts_"..type);
					tinsert(db, (#db+1), x);
					-- MiniMount_Msg("Inserting Mount #"..x..", the "..MountName..", MountSID: "..mountid..", into MiniMount_Mounts_"..type)
				end
			end
		end

		local count1, count2, count3, count4, count5 = 0,0,0,0,0;

		for z=1,#MiniMount_Mounts_Slow do
			if MiniMount_Mounts_Slow[z] == x then
				count1 = count1 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_Regular do
			if MiniMount_Mounts_Regular[z] == x then
				count2 = count2 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_Epic do
			if MiniMount_Mounts_Epic[z] == x then
				count3 = count3 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_FlyRegular do
			if MiniMount_Mounts_FlyRegular[z] == x then
				count4 = count4 + 1;
			end
		end
		for z=1,#MiniMount_Mounts_FlyEpic do
			if MiniMount_Mounts_FlyEpic[z] == x then
				count5 = count5 + 1;
			end
		end

		if (count1==0) and (count2==0) and (count3==0) and (count4==0) and (count5==0) then
			tinsert(MiniMount_Mounts_Unknown, x);
		end
	end

	-- Now for item mounts
	-- ItemID for Magic Broom: 37011

	-- Error Reporting
	if #MiniMount_Mounts_Unknown > 0 then
		for y=1, #MiniMount_Mounts_Unknown do
			local _, MountName, MountSID = GetCompanionInfo("MOUNT",MiniMount_Mounts_Unknown[y]);
			if not MiniMount_ReportedMounts[MountSID] then
				Reported = false;
			end
		end
		if Reported == false then
			MiniMount_Msg(MINIMOUNT_UNKNOWNMOUNTSFOUND,"error");
			Reported = true;
		end
		for y=1, #MiniMount_Mounts_Unknown do
			local _, MountName, MountSID = GetCompanionInfo("MOUNT",MiniMount_Mounts_Unknown[y]);
			if not MiniMount_ReportedMounts[MountSID] then
				tinsert(MiniMount_ReportedMounts, MountSID, MountName);
				MiniMount_Msg(MINIMOUNT_REPORTMOUNT.."SpellID: "..MountSID.." = "..MountName,"error");
			end
		end
	end
end

function MiniMount_Macro()
	if GetNumCompanions("MOUNT") == 0 then
		return;
	end
	local _, MountName = GetCompanionInfo("MOUNT",NextMount);

	local text = "#showtooltip "..tostring(MountName).."\n/mm";

	local index = GetMacroIndexByName("MiniMount")
	if index == 0 then
		CreateMacro("MiniMount", 1, text, 0);
		MiniMount_Msg(MINIMOUNT_MACROCREATED);
	else
		EditMacro(index,"MiniMount",1,text);
	end
end

function MiniMount_GetRidingSkill()
	local num = GetNumSkillLines();

	-- Localization (Required part)
	local mm_riding = "Riding";
	if GetLocale() == "deDE" then
		mm_riding = "Reiten";
	elseif GetLocale() == "frFR" then
		mm_riding = "Equitation";
	elseif GetLocale() == "esES" then
		mm_riding = "Equitacin";
	elseif GetLocale() == "esMX" then
		mm_riding = "Equitacin";
	elseif GetLocale() == "koKR" then
		mm_riding = "??";
	elseif GetLocale() == "zhCN" then
		mm_riding = "??";
	elseif GetLocale() == "zhTW" then
		mm_riding = "??";
	elseif GetLocale() == "ruRU" then
		mm_riding = "???????";
	end

	for x=1,num do
		local skillName, isHeader, _, skillRank = GetSkillLineInfo(x);
		if isHeader == nil then
			if skillName == mm_riding then
				MiniMount_RidingSkill = skillRank;
			end
		end
	end
end

function MiniMount_MountPrepare()
	local _, Class = UnitClass("player");

	-- Remove Druid's Shapeshift
	if Class == "DRUID" then
		local stance = GetShapeshiftForm(True);
		if stance ~= 0 then
			local _,name = GetShapeshiftFormInfo(stance);
			CancelUnitBuff("player",name);
		end
	end
end

--== Helper Functions ==--

-- Let's us sort tables!
function MiniMount_pairsByKeys (t, f)
	local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a, f)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then return nil
			else return a[i], t[a[i]]
			end
		end
	return iter
end

-- Detect and color mounts you already know!
function MiniMount_MerchantColorWindow()

	-- Determine Number of Merchant Items
	local TotalItems = GetMerchantNumItems();

	for i=1, TotalItems do
		local itemlink = GetMerchantItemLink(i);
		if itemlink == nil then
			break;
		end
		local _, _, _, _, _, itemType, itemSubType = GetItemInfo(itemlink);
		if (itemType == MISCELLANEOUS) and (itemSubType == MOUNT) then
			local itemid = MiniMount_GetID(itemlink);
			local name, rank, arg1, arg2, arg3, arg4 = GetItemSpell(itemlink);
			--MiniMount_Msg("Spell Name=\""..tostring(name).."\", Rank=\""..tostring(rank).."\", arg1=\""..tostring(arg1).."\", arg2=\""..tostring(arg2).."\", arg3=\""..tostring(arg3).."\", arg4=\""..tostring(arg4).."\"");

			--local spelllink = GetSpellLink(name, "");
			--local spellid = MiniMount_GetID(spelllink);

			--MiniMount_Msg("Mount SpellID = "..tostring(spellid));
		end
	end
end

-- mode 0 = base item ID, anything else = Full item ID
function MiniMount_GetID(itemlink)
	if type(itemlink) ~= "string" then
		return;
	end
	if (strfind(itemlink,":") == nil) then
		return;
	end

	local justItemId = string.gsub(itemlink,".-\124H([^\124]*)\124h.*", "%1");

	return itemid;
end

-- Timer function
function MiniMount_Timer(name,targettime,functiontodo)
	local framename = "MiniMountFrame"..name;
	if getglobal(framename) and (getglobal(framename):GetScript("OnUpdate") ~= nil) then
		return;
	end

	local frame = CreateFrame("Frame", framename);
	
	local totalElapsed = 0.0;
	local tickcount = 0;

	local function MiniMount_onUpdate(self, elapsed)
		totalElapsed = totalElapsed + elapsed;
		if (totalElapsed < 1) then return; end
		totalElapsed = totalElapsed - floor(totalElapsed);
		tickcount = tickcount + 1;

		if (tickcount == targettime) then
			functiontodo();
			frame:SetScript("OnUpdate", nil);
			return;
		end
		-- Enable for timing debugging
		-- ChatFrame1:AddMessage("tick... "..tickcount);
	end
	frame:SetScript("OnUpdate", MiniMount_onUpdate);
	frame:Show();
end